# -*- encoding : utf-8 -*-
module ThreeD
module ClassMethodCache

  def self.included(base)
    base.extend(ClassMethods)
  end


  module ClassMethods

    def activate_method_cache(methods_list = {})

      if !self.method_cache_enabled?

        cattr_accessor  :cached_methods
        cattr_accessor  :cache_storage

        # Setting up Cache Storage Variable
        cache_storage_name = "$#{self.name.underscore.upcase}_METHOD_CACHE"
        eval("#{cache_storage_name} = {}") 
        self.cache_storage = eval(cache_storage_name)


        self.add_cache_methods(methods_list)


        # Instanzmethoden für den User laden
        self.send :include,  InstanceMethods

      end    
    end  


    def add_cache_methods(methods_list = {})
      # Check cached methods list
      if methods_list.is_a?(Array)
        methods_list = {:default => methods_list}
      end

      if methods_list.is_a?(Hash)

        self.cached_methods ||= {}

        methods_list.each do |group, methods|
          if self.cached_methods[group.to_sym].nil?
            self.cached_methods[group.to_sym] = methods
          else
            self.cached_methods[group.to_sym] << methods
            self.cached_methods[group.to_sym] = self.cached_methods[group.to_sym].flatten
          end    
        end  
      else
        raise ArgumentError, "Wrong definition for cached methods: Use either an array (self.cached_methods will become {:default => array}) or a hash"
      end

      setup_cache_method_aliasing
    end  

    def setup_cache_method_aliasing
      # Setup des MethodAliasing
      self.instance_eval do
        self.cached_methods.each do |method_stack, methods|
          methods.each_with_index do |method, i|
            puts [self.name, method, self.instance_methods.include?(method.to_sym)].inspect
              
            # Methodennamen mit ?/!  am ende fixen - Alias anlegen
            needs_alias = false
            if method.to_s.last == "?"
              fixed_name = method.to_s.gsub("?","_with_soi")
              needs_alias = true
            elsif method.to_s.last == "!"
              fixed_name = method.to_s.gsub("?","_with_bang")
              needs_alias = true
            end
              
            if needs_alias
              alias_method fixed_name, method    
              
              method = fixed_name
            end
              
            define_method("#{method}_with_caching") do |*args|
              #begin
              start = Time.now
              # Key für den Cache erzeugen - Alle Parameter mit ablegen
              args_list = *args.flatten.map {|a| a.is_a?(ActiveRecord::Base) ? "#{a.class.name}-#{a.id}" : a}
              meth_name = [method,args_list].inspect.parameterize.to_s
              
              
              if cached?(meth_name)
                x = self.cached_attribute(meth_name)
              
                # => ActiveRecordArray disabled (03.08.2013, 18:54, Florian Eck)
                # if x.is_a?(ActiveRecordArray)
                #                    x = x.parse_record_data
                #                  end  
                CachingLog.info "  ... CALL cache_data #{meth_name} (#{Time.now-start})"
                return x
              else
                # Hier wird die Originalfunction aufgerufen
                orig = self.send("#{method}_without_caching", *args)
              
                # => ActiveRecordArray disabled
                #if orig.is_active_record_array?
                #parse = orig.parse_active_record_array
                #CachingLog.info "save ActiveRecordArray"
                #else
                parse = orig
                #end    
              
                v = cache_attribute!(meth_name, parse)
                CachingLog.info "  ... CREATE cache_data #{meth_name} (#{Time.now-start})"
                    
                return orig
                    
                    
              end
              #rescue Exception => e
              #  self.method_cache = {:last_error => e.backtrace, :last_error_class => e.class.inspect, :last_error_message => e.message}
              #  return nil #self.send("#{method}_without_caching", *args)
              #end     
              #__send__("#{method}_without_caching", *args)   
            end
              
            alias_method_chain method, :caching
          
          end
        end
      end
    end  

    def method_cache_enabled?
      self.included_modules.include?(InstanceMethods)
    end  
    
  end


  module InstanceMethods
    def method_cache
      m = self.class.cache_storage[self.id]
      if m.nil?
        # Wenn neue nutzer angelegt werden
        CachingLog.info "Create cache for #{self.class.name}(#{self.id})"
        self.class.cache_storage.merge!(self.id => {})
        self.class.cache_storage[self.id]
        return {}
      else
        return m
      end    
    end

    def method_cache=(val)
      self.class.cache_storage.merge!(self.id => val)
    end    

    def cached?(att)
      !self.method_cache[att.to_s].nil?  
    end

    def cached_attribute(att)
      self.method_cache[att.to_s]
    end

    def cache_attribute!(att, value)
      m = self.method_cache.merge!(att.to_s => value)
      #self.personal_status.update_attributes(:method_cache => m)
      # => not using DB anymore
      return value
    end

    # Clear methods.
    # Kann ein Methodenname, ein Array von Namen oder ein Regexp sein
    def clear_method_cache(meths = [], options = {})
      if meths.is_a?(Regexp)   
        meths = self.method_cache.keys.select {|k| !k.match(meths).nil? }
      elsif meths.is_a?(Symbol) || meths.is_a?(String)
        meths = self.class.cached_methods[meths.to_sym]  
      elsif !meths.is_a?(Array)
        meths = meths.to_a
      end  

      begin 
        c = self.method_cache
        meths_all = c.keys
        meths_valid = []
        meths.each do |m|
          meths_all.each do |mv|
            meths_valid << mv if mv.match(m)
          end  
        end  
        CachingLog.info meths_valid.inspect
        meths_valid.each do |m|
          CachingLog.info "DELETE cache data: #{m}"
          CachingLog.info  "   for #{self.personal_data.first_name} #{self.personal_data.last_name}"
          c.delete(m)
        end
      rescue
        # wenn was schief läuft: $user_method_caching für den user auf {} setzen
        CachingLog.info "RESET CACHE (forced): #{self.full_name} / #{self.id}"
        self.class.cache_storage[self.id] = {}
      end  
    end
    
    def reset_cache
      self.class.cache_storage[self.id] = {}
    end  

  end    


end  
end



